==> App-htmlsel-0.003.tar.gz: App-htmlsel-0.003/lib/App/htmlsel.pm <==
package App::htmlsel;

our $DATE = '2019-07-26'; # DATE
our $VERSION = '0.003'; # VERSION

use 5.010001;
use strict;
use warnings;

use App::CSelUtils;
use Scalar::Util qw(blessed refaddr);

our %SPEC;

$SPEC{htmlsel} = {
    v => 1.1,
    summary => 'Select HTML::Element nodes using CSel syntax',
    args => {
        %App::CSelUtils::foosel_common_args,
        %App::CSelUtils::foosel_tree_action_args,
    },
};
sub htmlsel {
    my %args = @_;

    my $expr = $args{expr};
    my $actions = $args{actions};

    # parse first so we can bail early on error without having to read the input
    require Data::CSel;
    Data::CSel::parse_csel($expr)
          or return [400, "Invalid CSel expression '$expr'"];

    require HTML::TreeBuilder;
    my $tree;
    if ($args{file} eq '-') {
        binmode STDIN, ":encoding(utf8)";
        $tree = HTML::TreeBuilder->new->parse_content(join "", <>);
    } else {
        #require File::Slurper;
        $tree = HTML::TreeBuilder->new->parse_file($args{file});
    }

    my $patch_handle;
  PATCH: {
        require Module::Patch;
        $patch_handle = Module::Patch::patch_package(
            'HTML::Element', [
                {
                    action   => 'add',
                    sub_name => 'children',
                    code     => sub {
                        my @children =
                            grep { blessed($_) && $_->isa('HTML::Element') }
                            @{ $_[0]{_content} };
                        #use DD; dd \@children;
                        @children;
                    },
                },
            ],
        );
    }

    my @matches = Data::CSel::csel(
        {class_prefixes=>['HTML']}, $expr, $tree);

     # skip root node itself to avoid duplication
    @matches = grep { refaddr($_) ne refaddr($tree) } @matches
        unless @matches <= 1;

    for my $action (@$actions) {
        if ($action eq 'print') {
            #$action = 'print_func_or_meth:meth:value.func:App::jsonsel::_encode_json';
            $action = 'print_method:as_HTML';
        }
    }

    App::CSelUtils::do_actions_on_nodes(
        nodes   => \@matches,
        actions => $args{actions},
    );
}

1;
# ABSTRACT: Select HTML::Element nodes using CSel syntax

__END__

=pod

=encoding UTF-8

=head1 NAME

App::htmlsel - Select HTML::Element nodes using CSel syntax

=head1 VERSION

This document describes version 0.003 of App::htmlsel (from Perl distribution App-htmlsel), released on 2019-07-26.

=head1 SYNOPSIS

=head1 FUNCTIONS


=head2 htmlsel

Usage:

 htmlsel(%args) -> [status, msg, payload, meta]

Select HTML::Element nodes using CSel syntax.

This function is not exported.

Arguments ('*' denotes required arguments):

=over 4

=item * B<actions> => I<array[str]> (default: ["print_as_string"])

Specify action(s) to perform on matching nodes.

=item * B<expr>* => I<str>

=item * B<file> => I<str> (default: "-")

=back

Returns an enveloped result (an array).

First element (status) is an integer containing HTTP status code
(200 means OK, 4xx caller error, 5xx function error). Second element
(msg) is a string containing error message, or 'OK' if status is
200. Third element (payload) is optional, the actual result. Fourth
element (meta) is called result metadata and is optional, a hash
that contains extra information.

Return value:  (any)

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/App-htmlsel>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-App-htmlsel>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=App-htmlsel>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 SEE ALSO

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2019 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

==> App-htmlsel-0.004.tar.gz: App-htmlsel-0.004/lib/App/htmlsel.pm <==
package App::htmlsel;

our $DATE = '2019-07-27'; # DATE
our $VERSION = '0.004'; # VERSION

use 5.010001;
use strict;
use warnings;

use App::CSelUtils;
use Scalar::Util qw(blessed refaddr);

our %SPEC;

$SPEC{htmlsel} = {
    v => 1.1,
    summary => 'Select HTML::Element nodes using CSel syntax',
    args => {
        %App::CSelUtils::foosel_common_args,
        %App::CSelUtils::foosel_tree_action_args,
    },
};
sub htmlsel {
    my %args = @_;

    my $expr = $args{expr};
    my $actions = $args{actions};

    # parse first so we can bail early on error without having to read the input
    require Data::CSel;
    Data::CSel::parse_csel($expr)
          or return [400, "Invalid CSel expression '$expr'"];

    require HTML::TreeBuilder;
    my $tree;
    if ($args{file} eq '-') {
        binmode STDIN, ":encoding(utf8)";
        $tree = HTML::TreeBuilder->new->parse_content(join "", <>);
    } else {
        #require File::Slurper;
        $tree = HTML::TreeBuilder->new->parse_file($args{file});
    }

    my $patch_handle;
  PATCH: {
        require Module::Patch;
        $patch_handle = Module::Patch::patch_package(
            'HTML::Element', [
                {
                    action   => 'add',
                    sub_name => 'children',
                    code     => sub {
                        my @children =
                            grep { blessed($_) && $_->isa('HTML::Element') }
                            @{ $_[0]{_content} };
                        #use DD; dd \@children;
                        @children;
                    },
                },
                {
                    action   => 'add',
                    sub_name => 'class',
                    code     => sub {
                        $_[0]{class};
                    },
                },
                {
                    action   => 'add',
                    sub_name => 'preview',
                    code     => sub {
                        my $res = $_[0]->as_HTML;
                        $res =~ s/\A\s+//s;
                        $res = substr($res, 0, 22)."..." if length($res) > 25;
                        $res =~ s/\n/ /g;
                        $res;
                    },
                },
            ],
        );
    }

    my @matches = Data::CSel::csel(
        {class_prefixes=>['HTML']}, $expr, $tree);

     # skip root node itself to avoid duplication
    @matches = grep { refaddr($_) ne refaddr($tree) } @matches
        unless @matches <= 1;

    for my $action (@$actions) {
        if ($action eq 'print' || $action eq 'print_as_string') {
            $action = 'print_method:as_HTML';
        } elsif ($action eq 'dump') {
            #$action = 'dump:tag.class.id';
            $action = 'dump:preview';
       }
    }

    App::CSelUtils::do_actions_on_nodes(
        nodes   => \@matches,
        actions => $args{actions},
    );
}

1;
# ABSTRACT: Select HTML::Element nodes using CSel syntax

__END__

=pod

=encoding UTF-8

=head1 NAME

App::htmlsel - Select HTML::Element nodes using CSel syntax

=head1 VERSION

This document describes version 0.004 of App::htmlsel (from Perl distribution App-htmlsel), released on 2019-07-27.

=head1 SYNOPSIS

=head1 FUNCTIONS


=head2 htmlsel

Usage:

 htmlsel(%args) -> [status, msg, payload, meta]

Select HTML::Element nodes using CSel syntax.

This function is not exported.

Arguments ('*' denotes required arguments):

=over 4

=item * B<actions> => I<array[str]> (default: ["print_as_string"])

Specify action(s) to perform on matching nodes.

=item * B<expr>* => I<str>

=item * B<file> => I<str> (default: "-")

=back

Returns an enveloped result (an array).

First element (status) is an integer containing HTTP status code
(200 means OK, 4xx caller error, 5xx function error). Second element
(msg) is a string containing error message, or 'OK' if status is
200. Third element (payload) is optional, the actual result. Fourth
element (meta) is called result metadata and is optional, a hash
that contains extra information.

Return value:  (any)

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/App-htmlsel>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-App-htmlsel>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=App-htmlsel>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 SEE ALSO

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2019 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
